--***************************************************************************
-- Copyright (c) 1994 - Ohio Board of Regents and
--                    - University of Cincinnati, Cincinnati, OH.
--                    - Sidhartha Mohanty,
--                    - Department of Electrical & Computer Engineering,
--                    - Sept, 1994.
--***************************************************************************
-- 
-- Title:               Design library for Queueing Model. Version 0.0.
-- 
-- File:                [SUITE: vhdl/project/queue] sink.vhdl
-- Created:             Sat Sept 17, 15:12:23 1994
-- Author:              Sidhartha Mohanty <smohanty@thor.ece.uc.edu>
-- 
-- Keywords:
-- LRM_version:         IEEE Std. P1076-1992/C
-- 
-- Objective:
--
-- Usage:
-- Analyze after queue packages and random package.
--
-- Algorithm:               
-- (None)
-- 
-- References:
-- 
-- Result:
-- 
-- History:
-- 
-- Comments:
-- 
--***************************************************************************

library queue;
use queue.queue_pkg.all;
use queue.rng.all;
use std.textio.all;

entity delay is
 
 generic ( delay_time       : time         := 1 fs;
	   name 	    : string       := "D1";
	   delay_dist       : distrib      := unif;
	   seed		    : integer      := 1;
	   generation       : frequency    := (1.0, 1.0);
	   debug_control    : debug        := none;
	   file_name 	    : string	   := "";
	   report_freq      : time         := 10 fs );

 port (in_arc : inout arc bus; out_arc : inout arc bus);

end;

architecture behavior of delay is

  signal number_serviced : integer := 0;
  signal id 		 : integer := 1;
  file outfile : text is out file_name;

begin

initialize : process

   variable integer_line : line;
   variable integer_id   : integer := 1;

  begin

    in_arc <= null;
    out_arc <= null;

    assert not std.textio.endfile(std.textio.input)
	report "(server.vhdl) Number of entities in Queueing Model exceed number of integer ids. Increase the number."
	severity failure;

    readline(std.textio.input, integer_line);

    assert integer_line'length >= 0
	report "(server.vhdl) Missing integer in identifiers file."
	severity failure;

    read(integer_line, integer_id);
    id <= integer_id;

    wait for 0 fs;

    wait;

end process;

delay: process

   variable unf  : Uniform := InitUniform(seed, generation.mean, generation.var);
   variable norml: Normal  := InitNormal(seed,generation.mean,generation.var);
   variable pois : Poisson := InitPoisson(seed, generation.mean);
   variable erlan: Erlang  := InitErlang(seed,generation.mean,generation.var);
   variable rnd  : Real    := 1.0;
   variable delay_time     : time := 1 fs;
   variable in_service     : integer := 0;
   variable mesg           : line;

 begin

   in_arc <= null;
   out_arc <= null;
   
   loop
    wait  until (in_arc.handshake = sending and  in_service = 0); 

    if (debug_control = handshaking or 
	debug_control = summary) then
    print_handshake_message("Delay Unit: ", name, " received handshake signal: ", 
			     in_arc.handshake,
                             " from component: ", in_arc.id,
			     " at Time: ", now);
    end if;

    if (delay_dist = unif) then
        assert false
          report "Delay Unit withs delay periods following uniform distribution"
          severity note;
	if (debug_control = trace_tokens or 
	    debug_control = summary) then

          write(mesg, string'("Delay Unit: "));
	  write(mesg, name);
	  write(mesg, string'(" with delay periods following UNIFORM distribution; Lower & Upper Bounds: "));
	  write(mesg, generation.mean);
	  write(mesg, string'(" "));
	  write(mesg, generation.var);
	  writeline(outfile, mesg);

          print_source_message("Delay Unit: ", name, " with delay periods following ",
                                delay_dist, "orm distribution; Lower & Upper bounds: ",
                                generation.mean, generation.var); 
       end if;
       GenRnd(unf);
       rnd := unf.rnd;
       delay_time := rnd * 1.0 fs;
    elsif (delay_dist = norm) then
        assert false
          report "Delay Unit with delay periods following  normal distribution"
          severity note;
        if (debug_control = trace_tokens or 
	    debug_control = summary) then

	  write(mesg, string'("Delay Unit: "));
	  write(mesg, name);
	  write(mesg, string'(" with delay periods following NORMAL distribution; Mean & Var: "));
	  write(mesg, generation.mean);
	  write(mesg, string'(" "));
	  write(mesg, generation.var);
	  writeline(outfile, mesg);

          print_source_message("Delay Unit: ", name, " with delay periods following ",
                                delay_dist, "al distribution; Mean & Var: ",
                                generation.mean, generation.var); 
        end if;
        GenRnd(norml);
        rnd := norml.rnd;
        delay_time := rnd * 1.0 fs;
    elsif (delay_dist = poiss) then
          assert false
            report "Delay Unit with delay periods following poisson distribution"
            severity note;
	  if (debug_control = trace_tokens or 
	      debug_control = summary) then

	  write(mesg, string'("Delay Unit: "));
	  write(mesg, name);
	  write(mesg, string'(" with delay periods following POISSON distribution; Mean: "));
	  write(mesg, generation.mean);
	  writeline(outfile, mesg);

          print_source_message("Delay Unit: ", name, " with delay periods following ",
                                delay_dist, "an distribution; Mean & Var: ",
                                generation.mean, generation.var); 
	  end if;
          GenRnd(pois);
          rnd := pois.rnd;
          delay_time := rnd * 1.0 fs;
    elsif (delay_dist = erl) then
          assert false
            report "Delay Unit with delay periods following binomial distribution"
            severity note;
	  if (debug_control = trace_tokens or 
	      debug_control = summary) then

	  write(mesg, string'("Delay Unit: "));
	  write(mesg, name);
	  write(mesg, string'(" with delay periods following ERLANG distribution; Mean & Var: "));
	  write(mesg, generation.mean);
	  write(mesg, string'(" "));
	  write(mesg, generation.var);
	  writeline(outfile, mesg);

          print_source_message("Delay Unit: ", name, " with delay periods following ",
                                delay_dist, "ang distribution; Mean & Var: ",
                                generation.mean, generation.var); 
	  end if;
          GenRnd(erlan);
          rnd := erlan.rnd;
          delay_time := rnd * 1.0 fs;
    elsif (delay_dist = fixed) then
          assert false
            report "Delay Unit with delay periods following fixed distribution"
            severity note;
	  if (debug_control = trace_tokens or 
	      debug_control = summary) then

	  write(mesg, string'("Delay Unit: "));
	  write(mesg, name);
	  write(mesg, string'(" with fixed delay periods: "));
	  write(mesg, generation.mean);
	  writeline(outfile, mesg);
          print_source_message("Delay Unit: ", name, " with delay periods following ",
                                delay_dist, " distribution; Mean & Var: ",
                                generation.mean, generation.var); 
	  end if;
          delay_time := delay_time; -- For General Dist., we use the default
	    			    -- generic defined service time.
    end if;

    wait for delay_time;
    out_arc <= in_arc;
    if (debug_control = handshaking or 
	debug_control = summary) then
    print_server_message ("Delay Unit: ", name, "  completed service for token from queue: ", 
	                   in_arc.id," at Time: ", now);
    end if;
    wait on out_arc until (out_arc.handshake = accepted);
    if (debug_control = handshaking or 
	debug_control = summary) then
    print_handshake_message("Delay Unit: ", name, " received handshake signal: ", 
	                     out_arc.handshake," from component: ", 
			     out_arc.id," at Time: ", now);
    end if;    
    in_arc <= (id, accepted, FALSE, now);
    wait for 0 fs;
    if (debug_control = handshaking or 
	debug_control = summary) then
    print_handshake_message ("Delay Unit: ", name, " sent handshake signal: ", 
	                      in_arc.handshake," to component: ", 
			      in_arc.id," at Time: ", now);
    end if;
    out_arc <= null;
    in_arc <= null;
    wait for 0 fs;

 end loop;

end process;

end behavior;
