--***************************************************************************
-- Copyright (c) 1994 - Ohio Board of Regents and 
--		      -	University of Cincinnati, Cincinnati, OH.
--		      - Sidhartha Mohanty, 
--		      -	Department of Electrical & Computer Engineering,
--		      - Sept, 1994.
--***************************************************************************
--
-- Title:		Design library for Queueing Model. Version 0.0.
--
-- File: 		[SUITE: vhdl/project/queue] queue_pkg.vhdl
-- Created: 		Mon Sept 12 10:28:09 1994
-- Author:		Sidhartha Mohanty <smohanty@thor.ece.uc.edu>
--
-- Keywords:
-- LRM_version: 	IEEE Std. P1076-1992/C
--
-- Objective:
--
-- Usage:
-- Analyze first.
--
-- Algorithm:
-- (None)
--
-- References:
--
-- Result:
--
-- History:
--
-- Comments:
--
--***************************************************************************

package queue_pkg is

type token_handshake is (ready, sending, accepted,
                            not_ready);

type queue_discipline is (FIFO, LIFO);

type service_type is (multi_token, single_token);

type token_priority is (P_ON, P_OFF);

type comp_type is (source_unit, join_unit, fork_unit, queue_unit,
				server_unit, sink_unit);

type source_name is (S0, S1, S2, S3, S4, S5, S6, S7, S8, S9, 
                      P1, P2, P3, P4, P5, P6, P7, P8, P9,
                       Q1, Q2, Q3, Q4, Q5, J1, J2, J3, J4, 
			J5, F1, F2, F3, F4, F5);

type bool_array is array (positive range <>) of boolean;

type probability_vector is array (positive range <>) of real;

type num_tokens_held is
 record
   held         : integer;
   change       : integer;
 end record;

type num_tokens is array (positive range <>) of num_tokens_held;

type num is array (positive range <>) of integer;

function resolve_num (args : num) return integer;

subtype tokens_change is resolve_num integer;

subtype tail_ptr is resolve_num integer;

function resolve_signal (drivers : bool_array) return boolean;

subtype token_avail is resolve_signal boolean;

type token is 
 record
   id    	: integer;
   handshake 	: token_handshake;
   tok_avail    : token_avail;
   t_stamp 	: time;
 end record;

type debug is (none, handshaking, trace_tokens, summary);

type token_array is array (natural range <>) of token;

type distrib is (unif, replicate, norm, poiss, erl, expo,
                  negex, random, fixed);

function resolve_arc (input_tokens : token_array) return token;

function resolve_bit (drivers : bit_vector) return bit;

subtype flow_cntrl is resolve_bit bit;

subtype arc is resolve_arc token;

type arc_array is array (natural range <>) of arc;

-- The inputs to frequency are different for the different types
-- of distributions and they should be specified accordingly or
-- default fixed values could be used instead.
-- For the Uniform distribution, the mean would be infact the
-- lower limit of the distribution and the var the upper limit.
-- For the Normal distribution they are what they mean exactly.
-- For the Poisson distribution the mean is specified and the var
-- value is not used. The mean has to be <= 1.0, and seed <= 1,
-- For the Fixed distribution, the mean is the average time
-- between the generation of two tokens, in fs.
-- All the values specified for mean and var should be >= 0.

type frequency is 
 record
   mean : real;
   var  : real;
 end record;
 
  function stringof (handshake : token_handshake) return string;
  function stringof (comp : comp_type) return string;
  function stringof (in_number : integer) return string;
  function stringof (token_time : time) return string;
  function stringof (dist : distrib) return string;
  function stringof (name : source_name) return string;

  procedure print_formatted_message (message : string);
  procedure print_formatted_message (num1, num2 : real);
  procedure print_handshake_message (message1 : string; name1 : string;
  				     message2 : string;
  				     handshake : token_handshake; 
  				     message3 : string;
  				     integer_id : integer;
  				     message4 : string;
  				     T : time);
  procedure print_formatted_message (message1 : string; T : time);
  procedure print_joinfork_message (message1 : string; name : string;
  				    message2 : string; message3 : string; 
  				    message4 : string; T : time);
  procedure print_queue_message (message1 : string; name1 : string;
  			         message2 : string; integer_id : integer;
  			         message3 : string; T: time);
  procedure print_server_message (message1 : string; name1 : string;
  			          message2 : string; integer_id : integer;
  			          message3 : string; T: time);
  procedure print_sink_message (message1 : string; name1 : string;
  			        message2 : string; integer_id : integer;
  			        message3 : string; T: time);
  procedure print_source_message (message1 : string; name : string;
  				  message2 : string; dist : distrib;
  				  message3 : string; 
  				  num1 : real; num2 : real);
  procedure print_source_message (message1 : string; name : string;
  				  message2 : string; dist : distrib;
  				  message3 : string; 
  				  num1 : integer; num2 : real);
  procedure print_token_message (message1 : string; message2 : string;
  			         T : time; message3 : string;
  			         num : integer);

end queue_pkg;
